/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.reuseextension.resource.rex.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractRexInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	private java.util.List<org.reuseware.coconut.reuseextension.resource.rex.IRexInterpreterListener> listeners = new java.util.ArrayList<org.reuseware.coconut.reuseextension.resource.rex.IRexInterpreterListener>();
	private org.eclipse.emf.ecore.EObject nextObjectToInterprete;
	private Object currentContext;
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		org.eclipse.emf.ecore.EObject next = null;
		currentContext = context;
		while (!interpretationStack.empty()) {
			try {
				next = interpretationStack.pop();
			} catch (java.util.EmptyStackException ese) {
				// this can happen when the interpreter was terminated between the call to empty()
				// and pop()
				break;
			}
			nextObjectToInterprete = next;
			notifyListeners(next);
			result = interprete(next, context);
			if (!continueInterpretation(context, result)) {
				break;
			}
		}
		currentContext = null;
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ContextType context, ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.reuseware.coconut.reuseextension.AnchorDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_AnchorDerivationRule((org.reuseware.coconut.reuseextension.AnchorDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.AddressablePointDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_AddressablePointDerivationRule((org.reuseware.coconut.reuseextension.AddressablePointDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.ComponentModelSpecification) {
			result = interprete_org_reuseware_coconut_reuseextension_ComponentModelSpecification((org.reuseware.coconut.reuseextension.ComponentModelSpecification) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding((org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.CompositionAssociationBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_CompositionAssociationBinding((org.reuseware.coconut.reuseextension.CompositionAssociationBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.CompositionLanguageSpecification) {
			result = interprete_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification((org.reuseware.coconut.reuseextension.CompositionLanguageSpecification) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification) {
			result = interprete_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification((org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding((org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.FragmentRoleBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_FragmentRoleBinding((org.reuseware.coconut.reuseextension.FragmentRoleBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.HookDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_HookDerivationRule((org.reuseware.coconut.reuseextension.HookDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortType2HeterogeneousPortBinding((org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortType2HomogenousPortBinding((org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortType2PortBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortType2PortBinding((org.reuseware.coconut.reuseextension.PortType2PortBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortType2SettingBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortType2SettingBinding((org.reuseware.coconut.reuseextension.PortType2SettingBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortType2SyntaxBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding((org.reuseware.coconut.reuseextension.PortType2SyntaxBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PortTypeBinding) {
			result = interprete_org_reuseware_coconut_reuseextension_PortTypeBinding((org.reuseware.coconut.reuseextension.PortTypeBinding) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.PrototypeDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_PrototypeDerivationRule((org.reuseware.coconut.reuseextension.PrototypeDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.ReferencePointDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_ReferencePointDerivationRule((org.reuseware.coconut.reuseextension.ReferencePointDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.ReuseExtension) {
			result = interprete_org_reuseware_coconut_reuseextension_ReuseExtension((org.reuseware.coconut.reuseextension.ReuseExtension) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.RootElementContext) {
			result = interprete_org_reuseware_coconut_reuseextension_RootElementContext((org.reuseware.coconut.reuseextension.RootElementContext) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.RuleContext) {
			result = interprete_org_reuseware_coconut_reuseextension_RuleContext((org.reuseware.coconut.reuseextension.RuleContext) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.SettingDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_SettingDerivationRule((org.reuseware.coconut.reuseextension.SettingDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.SlotDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_SlotDerivationRule((org.reuseware.coconut.reuseextension.SlotDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.SyntaxDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_SyntaxDerivationRule((org.reuseware.coconut.reuseextension.SyntaxDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.ValueHookDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_ValueHookDerivationRule((org.reuseware.coconut.reuseextension.ValueHookDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule((org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.reuseware.coconut.reuseextension.VariationPointDerivationRule) {
			result = interprete_org_reuseware_coconut_reuseextension_VariationPointDerivationRule((org.reuseware.coconut.reuseextension.VariationPointDerivationRule) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_AddressablePointDerivationRule(org.reuseware.coconut.reuseextension.AddressablePointDerivationRule addressablePointDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_AnchorDerivationRule(org.reuseware.coconut.reuseextension.AnchorDerivationRule anchorDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_ComponentModelSpecification(org.reuseware.coconut.reuseextension.ComponentModelSpecification componentModelSpecification, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding compositionAssociation2CompositionLinkBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_CompositionAssociationBinding(org.reuseware.coconut.reuseextension.CompositionAssociationBinding compositionAssociationBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification compositionLanguageSpecification, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification compositionLanguageSyntaxSpecification, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding fragmentRole2FragmentBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding fragmentRole2FragmentInstanceBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding fragmentRole2FragmentReferenceBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding fragmentRole2SyntaxBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_FragmentRoleBinding(org.reuseware.coconut.reuseextension.FragmentRoleBinding fragmentRoleBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_HookDerivationRule(org.reuseware.coconut.reuseextension.HookDerivationRule hookDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortType2HeterogeneousPortBinding(org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding portType2HeterogeneousPortBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortType2HomogenousPortBinding(org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding portType2HomogenousPortBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortType2PortBinding(org.reuseware.coconut.reuseextension.PortType2PortBinding portType2PortBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortType2SettingBinding(org.reuseware.coconut.reuseextension.PortType2SettingBinding portType2SettingBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding portType2SyntaxBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PortTypeBinding(org.reuseware.coconut.reuseextension.PortTypeBinding portTypeBinding, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_PrototypeDerivationRule(org.reuseware.coconut.reuseextension.PrototypeDerivationRule prototypeDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_ReferencePointDerivationRule(org.reuseware.coconut.reuseextension.ReferencePointDerivationRule referencePointDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_ReuseExtension(org.reuseware.coconut.reuseextension.ReuseExtension reuseExtension, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_RootElementContext(org.reuseware.coconut.reuseextension.RootElementContext rootElementContext, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_RuleContext(org.reuseware.coconut.reuseextension.RuleContext ruleContext, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_SettingDerivationRule(org.reuseware.coconut.reuseextension.SettingDerivationRule settingDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_SlotDerivationRule(org.reuseware.coconut.reuseextension.SlotDerivationRule slotDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_SyntaxDerivationRule(org.reuseware.coconut.reuseextension.SyntaxDerivationRule syntaxDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_ValueHookDerivationRule(org.reuseware.coconut.reuseextension.ValueHookDerivationRule valueHookDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule valuePrototypeDerivationRule, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_reuseware_coconut_reuseextension_VariationPointDerivationRule(org.reuseware.coconut.reuseextension.VariationPointDerivationRule variationPointDerivationRule, ContextType context) {
		return null;
	}
	
	private void notifyListeners(org.eclipse.emf.ecore.EObject element) {
		for (org.reuseware.coconut.reuseextension.resource.rex.IRexInterpreterListener listener : listeners) {
			listener.handleInterpreteObject(element);
		}
	}
	
	/**
	 * Adds the given object to the interpretation stack. Attention: Objects that are
	 * added first, are interpret last.
	 */
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
	/**
	 * Adds the given collection of objects to the interpretation stack. Attention:
	 * Collections that are added first, are interpret last.
	 */
	public void addObjectsToInterprete(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		for (org.eclipse.emf.ecore.EObject object : objects) {
			addObjectToInterprete(object);
		}
	}
	
	/**
	 * Adds the given collection of objects in reverse order to the interpretation
	 * stack.
	 */
	public void addObjectsToInterpreteInReverseOrder(java.util.Collection<? extends org.eclipse.emf.ecore.EObject> objects) {
		java.util.List<org.eclipse.emf.ecore.EObject> reverse = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>(objects.size());
		reverse.addAll(objects);
		java.util.Collections.reverse(reverse);
		addObjectsToInterprete(reverse);
	}
	
	/**
	 * Adds the given object and all its children to the interpretation stack such
	 * that they are interpret in top down order.
	 */
	public void addObjectTreeToInterpreteTopDown(org.eclipse.emf.ecore.EObject root) {
		java.util.List<org.eclipse.emf.ecore.EObject> objects = new java.util.ArrayList<org.eclipse.emf.ecore.EObject>();
		objects.add(root);
		java.util.Iterator<org.eclipse.emf.ecore.EObject> it = root.eAllContents();
		while (it.hasNext()) {
			org.eclipse.emf.ecore.EObject eObject = (org.eclipse.emf.ecore.EObject) it.next();
			objects.add(eObject);
		}
		addObjectsToInterpreteInReverseOrder(objects);
	}
	
	public void addListener(org.reuseware.coconut.reuseextension.resource.rex.IRexInterpreterListener newListener) {
		listeners.add(newListener);
	}
	
	public boolean removeListener(org.reuseware.coconut.reuseextension.resource.rex.IRexInterpreterListener listener) {
		return listeners.remove(listener);
	}
	
	public org.eclipse.emf.ecore.EObject getNextObjectToInterprete() {
		return nextObjectToInterprete;
	}
	
	public java.util.Stack<org.eclipse.emf.ecore.EObject> getInterpretationStack() {
		return interpretationStack;
	}
	
	public void terminate() {
		interpretationStack.clear();
	}
	
	public Object getCurrentContext() {
		return currentContext;
	}
	
}
